"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = transformer;

var _path = require("path");

var _getJSExports = _interopRequireDefault(require("../util/getJSExports"));

var _jscodeshiftAddImports = _interopRequireDefault(require("jscodeshift-add-imports"));

// istanbul ignore next
if (process.env.NODE_ENV === 'test') {
  var resolve = require.resolve;

  require.resolve = function (source) {
    return resolve(source.replace(/^@material-ui\/core\/es/, '../../../material-ui/src'));
  };
}

function transformer(fileInfo, api, options) {
  var j = api.jscodeshift;
  var importModule = options.importModule || '@material-ui/core';
  var targetModule = options.targetModule || '@material-ui/core';
  var whitelist = (0, _getJSExports.default)(require.resolve("".concat(importModule, "/es"), {
    paths: [(0, _path.dirname)(fileInfo.path)]
  }));
  var printOptions = options.printOptions || {
    quote: 'single',
    trailingComma: true
  };
  var root = j(fileInfo.source);
  var importRegExp = new RegExp("^".concat(importModule, "/(?:[^/]+/)*([^/]+)$"));
  var resultSpecifiers = [];
  root.find(j.ImportDeclaration).forEach(function (path) {
    if (!path.node.specifiers.length) return;
    if (path.value.importKind && path.value.importKind !== 'value') return;
    var importPath = path.value.source.value;
    var match = importPath.match(importRegExp);
    if (!match) return;
    if (importPath.includes('internal/')) return;
    path.node.specifiers.forEach(function (specifier, index) {
      if (specifier.importKind && specifier.importKind !== 'value') return;
      if (specifier.type === 'ImportNamespaceSpecifier') return;
      var localName = specifier.local.name;

      switch (specifier.type) {
        case 'ImportNamespaceSpecifier':
          return;

        case 'ImportDefaultSpecifier':
          {
            var moduleName = match[1];
            if (!whitelist.has(moduleName)) return;
            resultSpecifiers.push(j.importSpecifier(j.identifier(moduleName), j.identifier(localName)));
            path.get('specifiers', index).prune();
            break;
          }

        case 'ImportSpecifier':
          if (!whitelist.has(specifier.imported.name)) return;
          resultSpecifiers.push(specifier);
          path.get('specifiers', index).prune();
          break;

        default:
          break;
      }
    });
    if (!path.node.specifiers.length) path.prune();
  });

  if (resultSpecifiers.length) {
    (0, _jscodeshiftAddImports.default)(root, j.importDeclaration(resultSpecifiers, j.stringLiteral(targetModule)));
  }

  return root.toSource(printOptions);
}